import dicomParse from '../utils/dicomParse'
import config from '../../config'
const fs = require('fs')
const path = require('path')
const Busboy = require('busboy')
const crypto = require('crypto')
const inspect = require('util').inspect
const options = config.upload || {}
  /** 同步创建文件目录 */
const _mkdirsSync = dirname => {
    if (fs.existsSync(dirname)) {
      return true
    } else {
      if (_mkdirsSync(path.dirname(dirname))) {
        fs.mkdirSync(dirname);
        return true
      }
    }
  }
  /** 获取上传文件的后缀名 */
const _getSuffixName = fileName => {
    const nameList = fileName.split('.');
    return nameList[nameList.length - 1]
  }
  /** 获取上传文件的医院ID */
const _getHospitalId = fileName => {
    const nameList = fileName.split('-');
    return nameList[0]
  }
  /**dicom解析 */
const parseDicom = async result => new Promise((resolve, reject) => {
    dicomParse(result.fileName).then((dataSet) => {
      resolve(dataSet)
    }).catch(err => {
      reject(err)
    })
  })
  /**上传文件到fastdfs */
const uploadFile2FastDfs = async(fdfs, filePath) => new Promise((resolve, reject) => {
  fdfs.upload(filePath).then(function(fileId) {
    resolve(fileId)
  }).catch(function(err) {
    reject(err)
  })
})

const OSS = require('ali-oss')
const client = new OSS({
  region: 'oss-cn-beijing',
  accessKeyId: 'LTAImzRGKfWaL7Vi',
  accessKeySecret: 'GjWdd2cdHtbQkhhnhSxNbw0QChKD98',
  bucket: 'zskk-dcm',
});
const uploadFile2AliOss = async(filePath, md5, hospitalId) => {
    let result = await client.put(hospitalId + '/' + md5, filePath);
    return { url: result.url.replace('http', 'dicomweb'), metadata: result.name };
  }
  // dicomweb://zskk-dcm.oss-cn-beijing.aliyuncs.com/73090001/d10378a36815fe3babd9b1e814b68fe6
  // new Promise((resolve, reject) => {
  //     let result = await client.put(hospitalId + '/' + md5, filePath);
  //     config.log(result, 'result', hospitalId + '/' + md5, filePath)
  //   })
  /**删除文件 */
const deteleFile = async filePath => new Promise((resolve, reject) => {
    fs.unlink(filePath, function() {
      resolve(true)
    })
  })
  /**获取文件md5 */
const getFileMD5 = async filePath => new Promise((resolve, reject) => {
    let rs = fs.createReadStream(filePath);
    let hash = crypto.createHash('md5');
    rs.on('data', hash.update.bind(hash));
    rs.on('end', function() {
      resolve(hash.digest('hex'))
    });
  })
  /** 上传文件 */
const doUpload = async ctx => new Promise((resolve, reject) => {
  let req = ctx.req;
  let res = ctx.res;

  let busboy = new Busboy({ headers: req.headers });

  //获取类型
  let fileType = options.fileType || 'common';
  let filePath = path.join(options.path || '.', fileType);
  let mkdirResult = _mkdirsSync(filePath);
  let results = []
  let resultsLength = 0
  let finishFlag = false
  busboy.on('file', function(fieldname, file, filename, encoding, mimetype) {
    let fileName = Math.random().toString(16).substr(2) + '.' + _getSuffixName(filename);
    let _uploadFilePath = path.join(filePath, fileName);
    let saveTo = path.join(_uploadFilePath);
    resultsLength++;
    file.pipe(fs.createWriteStream(saveTo));
    file.on('end', function() {
      let result = {
        'success': true,
        'fileName': fileName,
        'hospital': _getHospitalId(filename)
      };
      results.push(result);
      if (finishFlag && results.length == resultsLength) {
        // setTimeout(() => {
        resolve(results)
          // }, 10000);
      }
    })
  });
  busboy.on('field', function(fieldname, val, fieldnameTruncated, valTruncated) {
    console.log(`Field [${fieldname}]: value: ${inspect(val)}`)
  });
  busboy.on('finish', function() {
    if (resultsLength == results.length) {
      // setTimeout(() => {
      resolve(results)
        // }, 10000);
    } else {
      finishFlag = true
      console.log(`busboy finish : error: results长度不对`)
    }
  });
  busboy.on('error', function(err) {
    console.log(`busboy error : value: ${err}`)
    reject(err)
  });
  req.pipe(busboy)
})
export { parseDicom, uploadFile2FastDfs, deteleFile, getFileMD5, doUpload, uploadFile2AliOss }
